Package: tidyomics
Title: Easily install and load the tidyomics ecosystem
Version: 1.9.0
Authors@R: c(
    person(given = "Stefano",
           family = "Mangiola",
           role = c("aut", "cre"),
           email = "mangiolastefano@gmail.com",
           comment = c(ORCID = "0000-0001-7474-836X")),
    person(given = "Michael",
           family = "Love",
           role = c("aut"),
           email = "michaelisaiahlove@gmail.com",
           comment = c(ORCID = "0000-0001-8401-0545")),
    person(given = "William",
           family = "Hutchison",
           role = c("aut"),
           email = "hutchison.w@wehi.edu.au",
           comment = c(ORCID = "0009-0001-6242-4269"))
	   )
Description: The tidyomics ecosystem is a set of packages for ’omic data analysis that work 
    together in harmony; they share common data representations and API design, consistent with the
    tidyverse ecosystem. The tidyomics package is designed to make it easy to install and load core 
    packages from the tidyomics ecosystem with a single command.
License: MIT + file LICENSE
URL: https://github.com/tidyomics/tidyomics
BugReports: https://github.com/tidyomics/tidyomics/issues
Depends: R (>= 4.2)
Imports: tidySummarizedExperiment, tidySingleCellExperiment,
        tidySpatialExperiment, tidyseurat, plyranges, purrr, rlang,
        stringr, cli,
Suggests: utils, tidyr, dplyr, tibble, ggplot2, mockr (>= 0.2.0), knitr
        (>= 1.41), rmarkdown (>= 2.20), testthat (>= 3.1.6),
        nullranges, tidybulk, plyinteractions
VignetteBuilder: knitr
Biarch: true
biocViews: AssayDomain, Infrastructure, RNASeq, DifferentialExpression,
        GeneExpression, Normalization, Clustering, QualityControl,
        Sequencing, Transcription, Transcriptomics
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/tidyomics
git_branch: devel
git_last_commit: a7fbe5a
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:34:57 UTC; biocbuild
Author: Stefano Mangiola [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7474-836X>),
  Michael Love [aut] (ORCID: <https://orcid.org/0000-0001-8401-0545>),
  William Hutchison [aut] (ORCID:
    <https://orcid.org/0009-0001-6242-4269>)
Maintainer: Stefano Mangiola <mangiolastefano@gmail.com>

# Generated by roxygen2: do not edit by hand

S3method(print,tidyomics_conflicts)
export(tidyomics)
export(tidyomics_conflicts)
export(tidyomics_packages)
importFrom(cli,ansi_align)
importFrom(cli,cat_line)
importFrom(cli,col_blue)
importFrom(cli,col_cyan)
importFrom(cli,col_green)
importFrom(cli,col_red)
importFrom(cli,format_inline)
importFrom(cli,rule)
importFrom(cli,style_bold)
importFrom(cli,symbol)
importFrom(purrr,imap)
importFrom(purrr,keep)
importFrom(purrr,map)
importFrom(purrr,map2_chr)
importFrom(purrr,map_chr)
importFrom(purrr,set_names)
importFrom(rlang,inform)
importFrom(rlang,is_installed)
importFrom(stringr,str_remove)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,stack)

<!-- README.md is generated from README.Rmd. Please edit that file -->

## Overview

The [tidyomics](https://github.com/tidyomics) ecosystem is a set of
packages for omics data analysis that work together in harmony; they
share common Bioconductor data representations and API design consistent
with the [tidyverse](https://www.tidyverse.org/) ecosystem. The
**tidyomics** package is designed to make it easy to install and load
core packages from the *tidyomics* ecosystem with a single command.

The core packages are:

``` r
tidyomics_packages()
```

    #  [1] "tidySummarizedExperiment" "tidySingleCellExperiment"
    #  [3] "tidyseurat"               "plyranges"               
    #  [5] "tidySpatialExperiment"

## The tidyomics ecosystem

### Core packages

The core packages are automatically loaded when you run
`library(tidyomics)`:

| Package | Intro | GitHub | Description |
|----|----|----|----|
| [tidySummarizedExperiment](https://stemangiola.github.io/tidySummarizedExperiment/) | [Vignette](https://stemangiola.github.io/tidySummarizedExperiment/articles/introduction.html) | [GitHub](https://github.com/stemangiola/tidySummarizedExperiment) | Tidy manipulation of SummarizedExperiment objects |
| [tidySingleCellExperiment](https://stemangiola.github.io/tidySingleCellExperiment) | [Vignette](https://stemangiola.github.io/tidySingleCellExperiment/articles/introduction.html) | [GitHub](https://github.com/stemangiola/tidySingleCellExperiment) | Tidy manipulation of SingleCellExperiment objects |
| [tidySeurat](https://stemangiola.github.io/tidyseurat/) | [Vignette](https://stemangiola.github.io/tidyseurat/articles/introduction.html) | [GitHub](https://github.com/stemangiola/tidyseurat) | Tidy manipulation of Seurat objects |
| [tidySpatialExperiment](https://william-hutchison.github.io/tidySpatialExperiment/) | [Vignette](https://william-hutchison.github.io/tidySpatialExperiment/articles/overview.html) | [GitHub](https://github.com/william-hutchison/tidySpatialExperiment) | Tidy manipulation of SpatialExperiment objects |
| [plyranges](https://sa-lee.github.io/plyranges/) | [Vignette](https://sa-lee.github.io/plyranges/articles/an-introduction.html) | [GitHub](https://github.com/sa-lee/plyranges) | Tidy manipulation of genomics ranges |

### Additional manipulation packages

| Package | Intro | GitHub | Description |
|----|----|----|----|
| [plyinteractions](https://tidyomics.github.io/plyinteractions/) | [Vignette](https://tidyomics.github.io/plyinteractions/articles/plyinteractions.html) | [GitHub](https://github.com/tidyomics/plyinteractions) | Tidy manipulation of genomic interactions |

**plyinteractions** is under evaluation for inclusion in the core
packages in future Bioconductor releases.

### Analysis packages

| Package | Intro | GitHub | Description |
|----|----|----|----|
| [tidybulk](https://stemangiola.github.io/tidybulk/) | [Vignette](https://stemangiola.github.io/tidybulk/articles/introduction.html) | [GitHub](https://github.com/stemangiola/tidybulk/) | Tidy bulk RNA-seq data analysis |
| [nullranges](https://nullranges.github.io/nullranges/) | [Vignette](https://nullranges.github.io/nullranges/articles/nullranges.html) | [GitHub](https://github.com/nullranges/nullranges/) | Generation of null genomic range sets |

## Installation

Installing the **tidyomics** package will install all core packages of
the *tidyomics* ecosystem. The **tidyomics** package can be installed
from Bioconductor:

``` r
BiocManager::install("tidyomics")
```

Additional packages can be installed independently as needed:

``` r
# Additional manipulation package
BiocManager::install("plyinteractions")

# Analysis packages
BiocManager::install("tidybulk")
BiocManager::install("nullranges")
```

## Loading the *tidyomics* ecosystem

The core *tidyomics* packages can be attached with:

``` r
library(tidyomics)
```

This command also produces a summary of package versions and function
conflicts. Function conflicts are a point of ongoing development and
will be addressed over time.

You are now ready to start using the *tidyomics* ecosystem.

#' Conflicts between the tidyomics and other packages
#'
#' This function lists all the conflicts between packages in the tidyomics
#' and other packages that you have loaded.
#'
#' There are four conflicts that are deliberately ignored: \code{intersect},
#' \code{union}, \code{setequal}, and \code{setdiff} from dplyr. These functions
#' make the base equivalents generic, so shouldn't negatively affect any
#' existing code.
#' 
#' @param only Set this to a character vector to restrict to conflicts only
#'   with these packages.
#' @return All conflicts between tidyomics packages and other packages that you have loaded.
#' @examples
#' tidyomics_conflicts()
#' 
#' @importFrom purrr set_names
#' @importFrom purrr keep
#' @importFrom purrr imap
#' @importFrom purrr set_names
#' @importFrom stringr str_remove
#' @export
tidyomics_conflicts <- function(only = NULL) {
  envs <- grep("^package:", search(), value = TRUE)
  envs <- purrr::set_names(envs)

  if (!is.null(only)) {
    only <- union(only, core)
    envs <- envs[names(envs) %in% paste0("package:", only)]
  }
  
  objs <- invert(lapply(envs, ls_env))

  conflicts <- purrr::keep(objs, ~ length(.x) > 1)

  tidy_names <- paste0("package:", tidyomics_packages())
  conflicts <- purrr::keep(conflicts, ~ any(.x %in% tidy_names))

  conflict_funs <- purrr::imap(conflicts, confirm_conflict)
  conflict_funs <- purrr::compact(conflict_funs)

  structure(conflict_funs, class = "tidyomics_conflicts")
}

#' Generate conflict message
#' 
#' @importFrom cli rule
#' @importFrom cli style_bold
#' @importFrom cli col_blue
#' @importFrom cli col_green
#' @importFrom cli col_red
#' @importFrom cli col_cyan
#' @importFrom cli symbol
#' @importFrom cli format_inline
#' @importFrom purrr map
#' @importFrom purrr map_chr
#' @importFrom purrr map2_chr
#' @noRd
tidyomics_conflict_message <- function(x) {
  header <- cli::rule(
    left = cli::style_bold("Conflicts"),
    right = "tidyomics_conflicts()"
  )

  pkgs <- x |> purrr::map(~ gsub("^package:", "", .))
  others <- pkgs |> purrr::map(`[`, -1)
  other_calls <- purrr::map2_chr(
    others, names(others),
    ~ paste0(cli::col_blue(.x), "::", .y, "()", collapse = ", ")
  )

  winner <- pkgs |> purrr::map_chr(1)
  funs <- format(paste0(cli::col_blue(winner), "::", cli::col_green(paste0(names(x), "()"))))
  bullets <- paste0(
    cli::col_red(cli::symbol$cross), " ", funs, " masks ", other_calls,
    collapse = "\n"
  )

  conflicted <- paste0(
    cli::col_cyan(cli::symbol$info), " ",
    cli::format_inline("Use the {.href [conflicted package](http://conflicted.r-lib.org/)} to force all conflicts to become errors"
  ))

  paste0(
    header, "\n",
    bullets, "\n",
    conflicted
  )
}

#' @importFrom cli cat_line
#' @export
print.tidyomics_conflicts <- function(x, ..., startup = FALSE) {
  cli::cat_line(tidyomics_conflict_message(x))
  invisible(x)
  }

#' @importFrom purrr map
#' @importFrom purrr keep
#' @noRd
confirm_conflict <- function(packages, name) {
  # Only look at functions
  objs <- packages |>
    purrr::map(~ get(name, pos = .)) |>
    purrr::keep(is.function)

  if (length(objs) <= 1)
    return()

  # Remove identical functions
  objs <- objs[!duplicated(objs)]
  packages <- packages[!duplicated(packages)]
  if (length(objs) == 1)
    return()

  packages
}

#' @noRd
ls_env <- function(env) {
  x <- ls(pos = env)

  # intersect, setdiff, setequal, union come from generics
  if (env %in% c("package:dplyr", "package:lubridate")) {
    x <- setdiff(x, c("intersect", "setdiff", "setequal", "union"))
  }

  if (env == "package:lubridate") {
    x <- setdiff(x, c(
      "as.difftime", # lubridate makes into an S4 generic
      "date"         # matches base behaviour
    ))
  }

  x
}

#' @noRd
.onAttach <- function(...) {
  attached <- tidyomics_attach()
  if (!is_loading_for_tests()) {
    inform_startup(tidyomics_attach_message(attached))
  }

  if (!is_attached("conflicted") && !is_loading_for_tests()) {
    conflicts <- tidyomics_conflicts()
    inform_startup(tidyomics_conflict_message(conflicts))
  }
}

#' @noRd
is_attached <- function(x) {
  paste0("package:", x) %in% search()
}

#' @noRd
is_loading_for_tests <- function() {
  !interactive() && identical(Sys.getenv("DEVTOOLS_LOAD"), "tidyomics")
}

#' @importFrom rlang inform
#' @noRd
inform_startup <- function(msg, ...) {
  if (is.null(msg)) {
    return()
  }
  if (isTRUE(getOption("tidyomics.quiet"))) {
    return()
  }

  rlang::inform(msg, ..., class = "packageStartupMessage")
}

#' List all packages loaded by the tidyomics metapackage
#'
#' @param include_self Include tidyomics in the list?
#' @return All packages loaded by the tidyomics metapackage.
#' @examples
#' tidyomics_packages()
#' 
#' @importFrom utils packageDescription
#' @export
tidyomics_packages <- function(include_self = TRUE) {
  
  names <- c("tidySummarizedExperiment", "tidySingleCellExperiment", "tidyseurat", "plyranges", "tidySpatialExperiment")
  
  if (include_self) {
    names <- c(names, "tidyomics")
  }

  names
}

#' @importFrom utils stack
#' @noRd
invert <- function(x) {
  if (length(x) == 0) return()
  stacked <- utils::stack(x)
  tapply(as.character(stacked$ind), stacked$values, list)
}

#' @importFrom rlang is_installed
#' @importFrom utils packageVersion
#' @noRd
packageVersion <- function(pkg) {
  if (rlang::is_installed(pkg)) {
    utils::packageVersion(pkg)
  } else {
    0
  }
}

core <- 
  c(
    
    # transcriptomics
    "tidySummarizedExperiment", "tidySingleCellExperiment", "tidySpatialExperiment", "tidyseurat", 
    
    # genomics
    "plyranges"
  )

#' @noRd
core_unloaded <- function() {
  search <- paste0("package:", core)
  core[!search %in% search()]
}

#' @noRd
same_library <- function(pkg) {
  loc <- if (pkg %in% loadedNamespaces()) dirname(getNamespaceInfo(pkg, "path"))
  library(pkg, lib.loc = loc, character.only = TRUE, warn.conflicts = FALSE)
}

 #' @noRd
tidyomics_attach <- function() {
  to_load <- core_unloaded()

  suppressPackageStartupMessages(
    lapply(to_load, same_library)
  )

  invisible(to_load)
}

#' @importFrom cli rule
#' @importFrom cli style_bold
#' @importFrom cli col_green
#' @importFrom cli ansi_align
#' @noRd
tidyomics_attach_message <- function(to_load) {
  if (length(to_load) == 0) {
    return(NULL)
  }

  header <- cli::rule(
    left = cli::style_bold("Attaching core tidyomics packages"),
    right = paste0("tidyomics ", package_version_h("tidyomics"))
  )

  to_load <- sort(to_load)
  versions <- vapply(to_load, package_version_h, character(1))

  packages <- paste0(
    cli::col_green(cli::symbol$tick), " ", cli::col_blue(format(to_load)), " ",
    cli::ansi_align(versions, max(cli::ansi_nchar(versions)))
  )

  if (length(packages) %% 2 == 1) {
    packages <- append(packages, "")
  }
  col1 <- seq_len(length(packages) / 2)
  info <- paste0(packages[col1], "     ", packages[-col1])

  paste0(header, "\n", paste(info, collapse = "\n"))
}

#' @importFrom utils packageVersion
#' @noRd
package_version_h <- function(pkg) {
  highlight_version(utils::packageVersion(pkg))
}

#' @importFrom cli col_red
#' @noRd
highlight_version <- function(x) {
  x <- as.character(x)

  is_dev <- function(x) {
    x <- suppressWarnings(as.numeric(x))
    !is.na(x) & x >= 9000
  }

  pieces <- strsplit(x, ".", fixed = TRUE)
  pieces <- lapply(pieces, function(x) ifelse(is_dev(x), cli::col_red(x), x))
  vapply(pieces, paste, collapse = ".", FUN.VALUE = character(1))
}

test_that("if no packages, shows nothing", {
  expect_snapshot(cat(tidyomics_attach_message(character())))
})

test_that("message lists all core tidyomics packages", {
  mockr::local_mock(package_version_h = function(x) "1.0.0")
  expect_snapshot(cat(tidyomics_attach_message(core)))
})

test_that("highlights dev versions in red", {
  local_reproducible_output(crayon = TRUE)

  expect_snapshot({
    highlight_version(c("1.0.0", "1.0.0.9000", "0.9000.0.9000", "1.0.0-rc"))
  })
})

#' Suppress R CMD check note
#' @noRd
ignore_unused_imports <- function() {
  tidySummarizedExperiment::unnest_summarized_experiment
  tidySingleCellExperiment::unnest_single_cell_experiment
  tidyseurat::unnest_seurat
  tidybulk::aggregate_duplicates
  plyranges::add_nearest_distance
  nullranges::focal
}

#' @noRd
release_bullets <- function() {
  c(
    '`usethis::use_latest_dependencies(TRUE, "CRAN")`',
    '`tidyomics_dependency_dissuade()`'
  )
}

#' tidyomics
#' 
#' @description A helper package to install and attach the core packages of the tidyomics ecosystem.
#' @export 
#' @return No value is returned
#' @examples
#' \dontrun{
#' # Install the core packages of the tidyomics ecosystem
#' BiocManager::install("tidyomics")
#' 
#' # Attach the core packages of the tidyomics ecosystem
#' library(tidyomics)
#' }
tidyomics <-
  function() {
    return(NULL)
  }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conflicts.R
\name{tidyomics_conflicts}
\alias{tidyomics_conflicts}
\title{Conflicts between the tidyomics and other packages}
\usage{
tidyomics_conflicts(only = NULL)
}
\arguments{
\item{only}{Set this to a character vector to restrict to conflicts only
with these packages.}
}
\value{
All conflicts between tidyomics packages and other packages that you have loaded.
}
\description{
This function lists all the conflicts between packages in the tidyomics
and other packages that you have loaded.
}
\details{
There are four conflicts that are deliberately ignored: \code{intersect},
\code{union}, \code{setequal}, and \code{setdiff} from dplyr. These functions
make the base equivalents generic, so shouldn't negatively affect any
existing code.
}
\examples{
tidyomics_conflicts()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{tidyomics_packages}
\alias{tidyomics_packages}
\title{List all packages loaded by the tidyomics metapackage}
\usage{
tidyomics_packages(include_self = TRUE)
}
\arguments{
\item{include_self}{Include tidyomics in the list?}
}
\value{
All packages loaded by the tidyomics metapackage.
}
\description{
List all packages loaded by the tidyomics metapackage
}
\examples{
tidyomics_packages()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidyomics.R
\name{tidyomics}
\alias{tidyomics}
\title{tidyomics}
\usage{
tidyomics()
}
\value{
No value is returned
}
\description{
A helper package to install and attach the core packages of the tidyomics ecosystem.
}
\examples{
\dontrun{
# Install the core packages of the tidyomics ecosystem
BiocManager::install("tidyomics")

# Attach the core packages of the tidyomics ecosystem
library(tidyomics)
}
}
